@@ -88,73 +88,55 @@ typedef struct bc_struct
 
 /* Function Prototypes */
 
-/* Define the _PROTOTYPE macro if it is needed. */
+void bc_init_numbers(void);
 
-#ifndef _PROTOTYPE
-#if defined(__STDC__) || defined(PHP_WIN32) && defined(__clang__)
-#define _PROTOTYPE(func, args) func args
-#else
-#define _PROTOTYPE(func, args) func()
-#endif
-#endif
+bc_num _bc_new_num_ex(int length, int scale, int persistent);
 
-_PROTOTYPE(void bc_init_numbers, (void));
+void _bc_free_num_ex(bc_num *num, int persistent);
 
-_PROTOTYPE(bc_num _bc_new_num_ex, (int length, int scale, int persistent));
+bc_num bc_copy_num(bc_num num);
 
-_PROTOTYPE(void _bc_free_num_ex, (bc_num *num, int persistent));
+void bc_init_num(bc_num *num);
 
-_PROTOTYPE(bc_num bc_copy_num, (bc_num num));
+int bc_str2num(bc_num *num, char *str, int scale);
 
-_PROTOTYPE(void bc_init_num, (bc_num *num));
+zend_string *bc_num2str_ex(bc_num num, int scale);
 
-_PROTOTYPE(int bc_str2num, (bc_num *num, char *str, int scale));
+void bc_int2num(bc_num *num, int val);
 
-_PROTOTYPE(zend_string *bc_num2str_ex, (bc_num num, int scale));
+long bc_num2long(bc_num num);
 
-_PROTOTYPE(void bc_int2num, (bc_num *num, int val));
+int bc_compare(bc_num n1, bc_num n2);
 
-_PROTOTYPE(long bc_num2long, (bc_num num));
+char bc_is_zero(bc_num num);
 
-_PROTOTYPE(int bc_compare, (bc_num n1, bc_num n2));
+char bc_is_zero_for_scale(bc_num num, int scale);
 
-_PROTOTYPE(char bc_is_zero, (bc_num num));
+char bc_is_near_zero(bc_num num, int scale);
 
-_PROTOTYPE(char bc_is_zero_for_scale, (bc_num num, int scale));
+char bc_is_neg(bc_num num);
 
-_PROTOTYPE(char bc_is_near_zero, (bc_num num, int scale));
+void bc_add(bc_num n1, bc_num n2, bc_num *result, int scale_min);
 
-_PROTOTYPE(char bc_is_neg, (bc_num num));
+void bc_sub(bc_num n1, bc_num n2, bc_num *result, int scale_min);
 
-_PROTOTYPE(void bc_add, (bc_num n1, bc_num n2, bc_num *result, int scale_min));
+void bc_multiply(bc_num n1, bc_num n2, bc_num *prod, int scale);
 
-_PROTOTYPE(void bc_sub, (bc_num n1, bc_num n2, bc_num *result, int scale_min));
+int bc_divide(bc_num n1, bc_num n2, bc_num *quot, int scale);
 
-_PROTOTYPE(void bc_multiply, (bc_num n1, bc_num n2, bc_num *prod, int scale));
+int bc_modulo(bc_num num1, bc_num num2, bc_num *resul, int scale);
 
-_PROTOTYPE(int bc_divide, (bc_num n1, bc_num n2, bc_num *quot, int scale));
+int bc_divmod(bc_num num1, bc_num num2, bc_num *quo, bc_num *rem, int scale);
 
-_PROTOTYPE(int bc_modulo, (bc_num num1, bc_num num2, bc_num *result,
-			   int scale));
+int bc_raisemod(bc_num base, bc_num expo, bc_num mo, bc_num *result, int scale);
 
-_PROTOTYPE(int bc_divmod, (bc_num num1, bc_num num2, bc_num *quot,
-			   bc_num *rem, int scale));
+void bc_raise(bc_num num1, bc_num num2, bc_num *resul, int scale);
 
-_PROTOTYPE(int bc_raisemod, (bc_num base, bc_num expo, bc_num mod,
-			     bc_num *result, int scale));
+int bc_sqrt(bc_num *num, int scale);
 
-_PROTOTYPE(void bc_raise, (bc_num num1, bc_num num2, bc_num *result,
-			   int scale));
-
-_PROTOTYPE(int bc_sqrt, (bc_num *num, int scale));
-
-_PROTOTYPE(void bc_out_num, (bc_num num, int o_base, void (* out_char)(int),
-			     int leading_zero));
+void bc_out_num(bc_num num, int o_base, void (* out_char)(char), int leading_zero);
 
 /* Prototypes needed for external utility routines. */
-
-_PROTOTYPE(void bc_out_of_memory, (void));
-
 #define bc_new_num(length, scale)	_bc_new_num_ex((length), (scale), 0)
 #define bc_free_num(num)			_bc_free_num_ex((num), 0)
 #define bc_num2str(num)				bc_num2str_ex((num), (num->n_scale))
