@@ -5,7 +5,7 @@
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -127,6 +127,18 @@ ZEND_GET_MODULE(oci8)
 
 #include "oci8_arginfo.h"
 
+static PHP_INI_MH(OnUpdateOldCloseSemantics)
+{
+	bool *p = (bool *) ZEND_INI_GET_ADDR();
+	*p = zend_ini_parse_bool(new_value);
+
+	if (*p) {
+		zend_error(E_DEPRECATED, "Directive oci8.old_oci_close_semantics is deprecated");
+	}
+
+	return SUCCESS;
+}
+
 /* {{{ extension definition structures */
 
 zend_module_entry oci8_module_entry = {
@@ -155,7 +167,7 @@ PHP_INI_BEGIN()
 	STD_PHP_INI_BOOLEAN("oci8.privileged_connect",		"0",	PHP_INI_SYSTEM,	OnUpdateBool,		privileged_connect,		zend_oci_globals,	oci_globals)
 	STD_PHP_INI_ENTRY(	"oci8.statement_cache_size",	"20",	PHP_INI_SYSTEM,	OnUpdateLong,	statement_cache_size,	zend_oci_globals,	oci_globals)
 	STD_PHP_INI_ENTRY(	"oci8.default_prefetch",		"100",	PHP_INI_SYSTEM,	OnUpdateLong,	default_prefetch,		zend_oci_globals,	oci_globals)
-	STD_PHP_INI_BOOLEAN("oci8.old_oci_close_semantics",	"0",	PHP_INI_SYSTEM,	OnUpdateBool,		old_oci_close_semantics,zend_oci_globals,	oci_globals)
+	STD_PHP_INI_BOOLEAN("oci8.old_oci_close_semantics",	"0",	PHP_INI_SYSTEM,	OnUpdateOldCloseSemantics,		old_oci_close_semantics,zend_oci_globals,	oci_globals)
 #if (OCI_MAJOR_VERSION >= 11)
 	STD_PHP_INI_ENTRY(	"oci8.connection_class",		"",		PHP_INI_ALL,	OnUpdateString,		connection_class,		zend_oci_globals,	oci_globals)
 #endif
@@ -239,24 +251,6 @@ static void php_oci_init_global_handles(void)
 }
 /* }}} */
 
-/* {{{ php_oci_cleanup_global_handles()
- *
- * Free global handles (if they were initialized before)
- */
-static void php_oci_cleanup_global_handles(void)
-{
-	if (OCI_G(err)) {
-		PHP_OCI_CALL(OCIHandleFree, ((dvoid *) OCI_G(err), OCI_HTYPE_ERROR));
-		OCI_G(err) = NULL;
-	}
-
-	if (OCI_G(env)) {
-		PHP_OCI_CALL(OCIHandleFree, ((dvoid *) OCI_G(env), OCI_HTYPE_ENV));
-		OCI_G(env) = NULL;
-	}
-}
-/* }}} */
-
 /* {{{ PHP_GINIT_FUNCTION
  *
  * Zerofill globals during module init
@@ -270,18 +264,28 @@ static PHP_GINIT_FUNCTION(oci)
 /* {{{ PHP_GSHUTDOWN_FUNCTION
  *
  * Called for thread shutdown in ZTS, after module shutdown for non-ZTS
+ * Free global handles (if they were initialized before)
  */
 static PHP_GSHUTDOWN_FUNCTION(oci)
 {
-	php_oci_cleanup_global_handles();
+	if (oci_globals->err) {
+		oci_globals->in_call = 1;
+		OCIHandleFree((dvoid *) oci_globals->err, OCI_HTYPE_ERROR);
+		oci_globals->in_call = 0;
+		oci_globals->err = NULL;
+	}
+
+	if (oci_globals->env) {
+		oci_globals->in_call = 1;
+		OCIHandleFree((dvoid *) oci_globals->env, OCI_HTYPE_ENV);
+		oci_globals->in_call = 0;
+		oci_globals->env = NULL;
+	}
 }
 /* }}} */
 
 PHP_MINIT_FUNCTION(oci)
 {
-	zend_class_entry oci_lob_class_entry;
-	zend_class_entry oci_coll_class_entry;
-
 	REGISTER_INI_ENTRIES();
 
 	le_statement = zend_register_list_destructors_ex(php_oci_statement_list_dtor, NULL, "oci8 statement", module_number);
@@ -291,11 +295,8 @@ PHP_MINIT_FUNCTION(oci)
 	le_descriptor = zend_register_list_destructors_ex(php_oci_descriptor_list_dtor, NULL, "oci8 descriptor", module_number);
 	le_collection = zend_register_list_destructors_ex(php_oci_collection_list_dtor, NULL, "oci8 collection", module_number);
 
-	INIT_CLASS_ENTRY(oci_lob_class_entry, "OCILob", class_OCILob_methods);
-	INIT_CLASS_ENTRY(oci_coll_class_entry, "OCICollection", class_OCICollection_methods);
-
-	oci_lob_class_entry_ptr = zend_register_internal_class(&oci_lob_class_entry);
-	oci_coll_class_entry_ptr = zend_register_internal_class(&oci_coll_class_entry);
+	oci_lob_class_entry_ptr = register_class_OCILob();
+	oci_coll_class_entry_ptr = register_class_OCICollection();
 
 /* thies@thieso.net 990203 i do not think that we will need all of them - just in here for completeness for now! */
 	REGISTER_LONG_CONSTANT("OCI_DEFAULT",OCI_DEFAULT, CONST_CS | CONST_PERSISTENT);
@@ -646,7 +647,7 @@ void php_oci_define_hash_dtor(zval *data)
 
 	zval_ptr_dtor(&define->val);
 
-    efree(define);
+	efree(define);
 }
 /* }}} */
 
@@ -923,8 +924,8 @@ php_oci_connection *php_oci_do_connect_ex(char *username, int username_len, char
 	smart_str hashed_details = {0};
 	time_t timestamp;
 	php_oci_spool *session_pool = NULL;
-	zend_bool use_spool = 1;	   /* Default is to use client-side session pool */
-	zend_bool ping_done = 0;
+	bool use_spool = 1;	   /* Default is to use client-side session pool */
+	bool ping_done = 0;
 
 	ub2 charsetid = 0;
 	ub2 charsetid_nls_lang = 0;
@@ -1039,10 +1040,10 @@ php_oci_connection *php_oci_do_connect_ex(char *username, int username_len, char
 	smart_str_0(&hashed_details);
 
 	/* make it lowercase */
-	php_strtolower(ZSTR_VAL(hashed_details.s), ZSTR_LEN(hashed_details.s));
+	zend_str_tolower(ZSTR_VAL(hashed_details.s), ZSTR_LEN(hashed_details.s));
 
 	if (!exclusive && !new_password) {
-		zend_bool found = 0;
+		bool found = 0;
 
 		if (persistent && ((zvp = zend_hash_find(&EG(persistent_list), hashed_details.s))) != NULL) {
 			zend_resource *le = Z_RES_P(zvp);
@@ -1190,7 +1191,7 @@ php_oci_connection *php_oci_do_connect_ex(char *username, int username_len, char
 	 * a last resort, return a non-persistent connection.
 	 */
 	if (persistent) {
-		zend_bool alloc_non_persistent = 0;
+		bool alloc_non_persistent = 0;
 
 		if (OCI_G(max_persistent) != -1 && OCI_G(num_persistent) >= OCI_G(max_persistent)) {
 			/* try to find an idle connection and kill it */
@@ -1436,7 +1437,7 @@ int php_oci_connection_commit(php_oci_connection *connection)
 static int php_oci_connection_close(php_oci_connection *connection)
 {
 	int result = 0;
-	zend_bool in_call_save = OCI_G(in_call);
+	bool in_call_save = OCI_G(in_call);
 
 #ifdef HAVE_OCI8_DTRACE
 	if (DTRACE_OCI8_CONNECTION_CLOSE_ENABLED()) {
@@ -1528,7 +1529,7 @@ static int php_oci_connection_close(php_oci_connection *connection)
 int php_oci_connection_release(php_oci_connection *connection)
 {
 	int result = 0;
-	zend_bool in_call_save = OCI_G(in_call);
+	bool in_call_save = OCI_G(in_call);
 	time_t timestamp = time(NULL);
 
 	if (connection->is_stub) {
@@ -1705,7 +1706,7 @@ int php_oci_column_to_zval(php_oci_out_column *column, zval *value, int mode)
 			descriptor = (php_oci_descriptor *) column->descid->ptr;
 
 			if (!descriptor) {
-				php_error_docref(NULL, E_WARNING, "Unable to find LOB descriptor #%d", column->descid->handle);
+				php_error_docref(NULL, E_WARNING, "Unable to find LOB descriptor #" ZEND_LONG_FMT, column->descid->handle);
 				return 1;
 			}
 
@@ -1891,7 +1892,7 @@ void php_oci_fetch_row (INTERNAL_FUNCTION_PARAMETERS, int mode, int expected_arg
 		} else {
 			RETURN_FALSE;
 		}
-    }
+	}
 #endif /* OCI_MAJOR_VERSION */
 
 	if (placeholder == NULL) {
@@ -1989,7 +1990,7 @@ static int php_oci_persistent_helper(zval *zv)
 static php_oci_spool *php_oci_create_spool(char *username, int username_len, char *password, int password_len, char *dbname, int dbname_len, zend_string *hash_key, int charsetid)
 {
 	php_oci_spool *session_pool = NULL;
-	zend_bool iserror = 0;
+	bool iserror = 0;
 	ub4 poolmode = OCI_DEFAULT;	/* Mode to be passed to OCISessionPoolCreate */
 	OCIAuthInfo *spoolAuth = NULL;
 	sword errstatus;
@@ -2052,7 +2053,7 @@ static php_oci_spool *php_oci_create_spool(char *username, int username_len, cha
 		OCI_G(errcode) = php_oci_error(OCI_G(err), errstatus);
 		iserror = 1;
 		goto exit_create_spool;
- 	}
+	}
 	/* }}} */
 
 	/* {{{ Set the edition attribute on the auth handle */
@@ -2128,7 +2129,7 @@ static php_oci_spool *php_oci_get_spool(char *username, int username_len, char *
 	smart_str spool_hashed_details = {0};
 	php_oci_spool *session_pool = NULL;
 	zend_resource *spool_out_le = NULL;
-	zend_bool iserror = 0;
+	bool iserror = 0;
 	zval *spool_out_zv = NULL;
 
 	/* {{{ Create the spool hash key */
@@ -2157,7 +2158,7 @@ static php_oci_spool *php_oci_get_spool(char *username, int username_len, char *
 	/* Session Pool Hash Key : oci8spool***username**edition**hashedpassword**dbname**charset */
 
 	smart_str_0(&spool_hashed_details);
-	php_strtolower(ZSTR_VAL(spool_hashed_details.s), ZSTR_LEN(spool_hashed_details.s));
+	zend_str_tolower(ZSTR_VAL(spool_hashed_details.s), ZSTR_LEN(spool_hashed_details.s));
 	/* }}} */
 
 	spool_out_zv = zend_hash_find(&EG(persistent_list), spool_hashed_details.s);
