@@ -53,7 +53,9 @@
 /* }}} */
 
 #include "ext/standard/php_string.h"
+ZEND_DIAGNOSTIC_IGNORED_START("-Wstrict-prototypes")
 #include <oci.h>
+ZEND_DIAGNOSTIC_IGNORED_END
 
 extern int le_connection;
 extern int le_pconnection;
