@@ -21,7 +21,7 @@
 
 #ifdef HAVE_CAPSTONE
 # define HAVE_DISASM 1
-# include <capstone/capstone.h>
+# include <capstone.h>
 # define HAVE_CAPSTONE_ITER 1
 #elif ZEND_JIT_TARGET_X86
 # define HAVE_DISASM 1
@@ -664,6 +664,7 @@ static int zend_jit_disasm_init(void)
 	REGISTER_HELPER(zend_jit_post_inc_typed_ref);
 	REGISTER_HELPER(zend_jit_post_dec_typed_ref);
 	REGISTER_HELPER(zend_jit_assign_op_to_typed_ref);
+	REGISTER_HELPER(zend_jit_assign_op_to_typed_ref_tmp);
 	REGISTER_HELPER(zend_jit_only_vars_by_reference);
 	REGISTER_HELPER(zend_jit_invalid_array_access);
 	REGISTER_HELPER(zend_jit_invalid_property_read);
