@@ -1,13 +1,11 @@
 /*
-  +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_01.txt                                  |
+  | https://www.php.net/license/3_01.txt                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -31,9 +29,434 @@
 #include "php_pdo_firebird.h"
 #include "php_pdo_firebird_int.h"
 
-static int firebird_alloc_prepare_stmt(pdo_dbh_t*, const char*, size_t, XSQLDA*, isc_stmt_handle*,
+static int firebird_alloc_prepare_stmt(pdo_dbh_t*, const zend_string*, XSQLDA*, isc_stmt_handle*,
 	HashTable*);
 
+const char CHR_LETTER = 1;
+const char CHR_DIGIT = 2;
+const char CHR_IDENT = 4;
+const char CHR_QUOTE = 8;
+const char CHR_WHITE = 16;
+const char CHR_HEX = 32;
+const char CHR_INTRODUCER = 64;
+
+static const char classes_array[] = {
+	/* 000     */ 0,
+	/* 001     */ 0,
+	/* 002     */ 0,
+	/* 003     */ 0,
+	/* 004     */ 0,
+	/* 005     */ 0,
+	/* 006     */ 0,
+	/* 007     */ 0,
+	/* 008     */ 0,
+	/* 009     */ 16, /* CHR_WHITE */
+	/* 010     */ 16, /* CHR_WHITE */
+	/* 011     */ 0,
+	/* 012     */ 0,
+	/* 013     */ 16, /* CHR_WHITE */
+	/* 014     */ 0,
+	/* 015     */ 0,
+	/* 016     */ 0,
+	/* 017     */ 0,
+	/* 018     */ 0,
+	/* 019     */ 0,
+	/* 020     */ 0,
+	/* 021     */ 0,
+	/* 022     */ 0,
+	/* 023     */ 0,
+	/* 024     */ 0,
+	/* 025     */ 0,
+	/* 026     */ 0,
+	/* 027     */ 0,
+	/* 028     */ 0,
+	/* 029     */ 0,
+	/* 030     */ 0,
+	/* 031     */ 0,
+	/* 032     */ 16, /* CHR_WHITE */
+	/* 033  !  */ 0,
+	/* 034  "  */ 8, /* CHR_QUOTE */
+	/* 035  #  */ 0,
+	/* 036  $  */ 4, /* CHR_IDENT */
+	/* 037  %  */ 0,
+	/* 038  &  */ 0,
+	/* 039  '  */ 8, /* CHR_QUOTE */
+	/* 040  (  */ 0,
+	/* 041  )  */ 0,
+	/* 042  *  */ 0,
+	/* 043  +  */ 0,
+	/* 044  ,  */ 0,
+	/* 045  -  */ 0,
+	/* 046  .  */ 0,
+	/* 047  /  */ 0,
+	/* 048  0  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 049  1  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 050  2  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 051  3  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 052  4  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 053  5  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 054  6  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 055  7  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 056  8  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 057  9  */ 38, /* CHR_DIGIT | CHR_IDENT | CHR_HEX */
+	/* 058  :  */ 0,
+	/* 059  ;  */ 0,
+	/* 060  <  */ 0,
+	/* 061  =  */ 0,
+	/* 062  >  */ 0,
+	/* 063  ?  */ 0,
+	/* 064  @  */ 0,
+	/* 065  A  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 066  B  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 067  C  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 068  D  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 069  E  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 070  F  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 071  G  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 072  H  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 073  I  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 074  J  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 075  K  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 076  L  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 077  M  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 078  N  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 079  O  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 080  P  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 081  Q  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 082  R  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 083  S  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 084  T  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 085  U  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 086  V  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 087  W  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 088  X  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 089  Y  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 090  Z  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 091  [  */ 0,
+	/* 092  \  */ 0,
+	/* 093  ]  */ 0,
+	/* 094  ^  */ 0,
+	/* 095  _  */ 68, /* CHR_IDENT | CHR_INTRODUCER */
+	/* 096  `  */ 0,
+	/* 097  a  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 098  b  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 099  c  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 100  d  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 101  e  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 102  f  */ 37, /* CHR_LETTER | CHR_IDENT | CHR_HEX */
+	/* 103  g  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 104  h  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 105  i  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 106  j  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 107  k  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 108  l  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 109  m  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 110  n  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 111  o  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 112  p  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 113  q  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 114  r  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 115  s  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 116  t  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 117  u  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 118  v  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 119  w  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 120  x  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 121  y  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 122  z  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 123  {  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 124  |  */ 0,
+	/* 125  }  */ 5, /* CHR_LETTER | CHR_IDENT */
+	/* 126  ~  */ 0,
+	/* 127     */ 0
+};
+
+static inline char classes(char idx)
+{
+	unsigned char uidx = (unsigned char) idx;
+	if (uidx > 127) return 0;
+	return classes_array[uidx];
+}
+
+typedef enum {
+	ttNone,
+	ttWhite,
+	ttComment,
+	ttBrokenComment,
+	ttString,
+	ttParamMark,
+	ttIdent,
+	ttOther
+} FbTokenType;
+
+static FbTokenType getToken(const char** begin, const char* end)
+{
+	FbTokenType ret = ttNone;
+	const char* p = *begin;
+
+	char c = *p++;
+	switch (c)
+	{
+	case ':':
+	case '?':
+		ret = ttParamMark;
+		break;
+
+	case '\'':
+	case '"':
+		while (p < end)
+		{
+			if (*p++ == c)
+			{
+				ret = ttString;
+				break;
+			}
+		}
+		break;
+
+	case '/':
+		if (p < end && *p == '*')
+		{
+			ret = ttBrokenComment;
+			p++;
+			while (p < end)
+			{
+				if (*p++ == '*' && p < end && *p == '/')
+				{
+					p++;
+					ret = ttComment;
+					break;
+				}
+			}
+		}
+		else {
+			ret = ttOther;
+		}
+		break;
+
+	case '-':
+		if (p < end && *p == '-')
+		{
+			while (++p < end)
+			{
+				if (*p == '\r')
+				{
+					p++;
+					if (p < end && *p == '\n')
+						p++;
+					break;
+				}
+				else if (*p == '\n')
+					break;
+			}
+
+			ret = ttComment;
+		}
+		else
+			ret = ttOther;
+		break;
+
+	default:
+		if (classes(c) & CHR_DIGIT)
+		{
+			while (p < end && (classes(*p) & CHR_DIGIT))
+				p++;
+			ret = ttOther;
+		}
+		else if (classes(c) & CHR_IDENT)
+		{
+			while (p < end && (classes(*p) & CHR_IDENT))
+				p++;
+			ret = ttIdent;
+		}
+		else if (classes(c) & CHR_WHITE)
+		{
+			while (p < end && (classes(*p) & CHR_WHITE))
+				p++;
+			ret = ttWhite;
+		}
+		else
+		{
+			while (p < end && !(classes(*p) & (CHR_DIGIT | CHR_IDENT | CHR_WHITE)) &&
+				(*p != '/') && (*p != '-') && (*p != ':') && (*p != '?') &&
+				(*p != '\'') && (*p != '"'))
+			{
+				p++;
+			}
+			ret = ttOther;
+		}
+	}
+
+	*begin = p;
+	return ret;
+}
+
+int preprocess(const zend_string* sql, char* sql_out, HashTable* named_params)
+{
+	bool passAsIs = 1, execBlock = 0;
+	zend_long pindex = -1;
+	char pname[254], ident[253], ident2[253];
+	unsigned int l;
+	const char* p = ZSTR_VAL(sql), * end = ZSTR_VAL(sql) + ZSTR_LEN(sql);
+	const char* start = p;
+	FbTokenType tok = getToken(&p, end);
+
+	const char* i = start;
+	while (p < end && (tok == ttComment || tok == ttWhite))
+	{
+		i = p;
+		tok = getToken(&p, end);
+	}
+
+	if (p >= end || tok != ttIdent)
+	{
+		/* Execute statement preprocess SQL error */
+		/* Statement expected */
+		return 0;
+	}
+	/* skip leading comments ?? */
+	start = i;
+	l = p - i;
+	/* check the length of the identifier */
+	/* in Firebird 4.0 it is 63 characters, in previous versions 31 bytes */
+	if (l > 252) {
+		return 0;
+	}
+	strncpy(ident, i, l);
+	ident[l] = '\0';
+	if (!strcasecmp(ident, "EXECUTE"))
+	{
+		/* For EXECUTE PROCEDURE and EXECUTE BLOCK statements, named parameters must be processed. */
+		/* However, in EXECUTE BLOCK this is done in a special way. */
+		const char* i2 = p;
+		tok = getToken(&p, end);
+		while (p < end && (tok == ttComment || tok == ttWhite))
+		{
+			i2 = p;
+			tok = getToken(&p, end);
+		}
+		if (p >= end || tok != ttIdent)
+		{
+			/* Execute statement preprocess SQL error */
+			/* Statement expected */
+			return 0;
+		}
+		l = p - i2;
+		/* check the length of the identifier */
+		/* in Firebird 4.0 it is 63 characters, in previous versions 31 bytes */
+		if (l > 252) {
+			return 0;
+		}
+		strncpy(ident2, i2, l);
+		ident2[l] = '\0';
+		execBlock = !strcasecmp(ident2, "BLOCK");
+		passAsIs = 0;
+	}
+	else
+	{
+		/* Named parameters must be processed in the INSERT, UPDATE, DELETE, MERGE statements. */
+		/* If CTEs are present in the query, they begin with the WITH keyword. */
+		passAsIs = strcasecmp(ident, "INSERT") && strcasecmp(ident, "UPDATE") &&
+			strcasecmp(ident, "DELETE") && strcasecmp(ident, "MERGE") &&
+			strcasecmp(ident, "SELECT") && strcasecmp(ident, "WITH");
+	}
+
+	if (passAsIs)
+	{
+		strcpy(sql_out, ZSTR_VAL(sql));
+		return 1;
+	}
+
+	strncat(sql_out, start, p - start);
+
+	while (p < end)
+	{
+		start = p;
+		tok = getToken(&p, end);
+		switch (tok)
+		{
+		case ttParamMark:
+			tok = getToken(&p, end);
+			if (tok == ttIdent /*|| tok == ttString*/)
+			{
+				++pindex;
+				l = p - start;
+				/* check the length of the identifier */
+				/* in Firebird 4.0 it is 63 characters, in previous versions 31 bytes */
+				/* + symbol ":" */
+				if (l > 253) {
+					return 0;
+				}
+				strncpy(pname, start, l);
+				pname[l] = '\0';
+
+				if (named_params) {
+					zval tmp;
+					ZVAL_LONG(&tmp, pindex);
+					zend_hash_str_update(named_params, pname, l, &tmp);
+				}
+
+				strcat(sql_out, "?");
+			}
+			else
+			{
+				if (strncmp(start, "?", 1)) {
+					/* Execute statement preprocess SQL error */
+					/* Parameter name expected */
+					return 0;
+				}
+				++pindex;
+				strncat(sql_out, start, p - start);
+			}
+			break;
+
+		case ttIdent:
+			if (execBlock)
+			{
+				/* In the EXECUTE BLOCK statement, processing must be */
+				/* carried out up to the keyword AS. */
+				l = p - start;
+				/* check the length of the identifier */
+				/* in Firebird 4.0 it is 63 characters, in previous versions 31 bytes */
+				if (l > 252) {
+					return 0;
+				}
+				strncpy(ident, start, l);
+				ident[l] = '\0';
+				if (!strcasecmp(ident, "AS"))
+				{
+					strncat(sql_out, start, end - start);
+					return 1;
+				}
+			}
+			/* TODO Check this is correct? */
+			ZEND_FALLTHROUGH;
+
+		case ttWhite:
+		case ttComment:
+		case ttString:
+		case ttOther:
+			strncat(sql_out, start, p - start);
+			break;
+
+		case ttBrokenComment:
+		{
+			/* Execute statement preprocess SQL error */
+			/* Unclosed comment found near ''@1'' */
+			return 0;
+		}
+		break;
+
+
+		case ttNone:
+			/* Execute statement preprocess SQL error */
+			return 0;
+			break;
+		}
+	}
+	return 1;
+}
+
 /* map driver specific error message to PDO error */
 void _firebird_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, char const *file, zend_long line) /* {{{ */
 {
@@ -46,7 +469,7 @@ void _firebird_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, char const *file, zend_lo
 #define RECORD_ERROR(dbh) _firebird_error(dbh, NULL, __FILE__, __LINE__)
 
 /* called by PDO to close a db handle */
-static int firebird_handle_closer(pdo_dbh_t *dbh) /* {{{ */
+static void firebird_handle_closer(pdo_dbh_t *dbh) /* {{{ */
 {
 	pdo_firebird_db_handle *H = (pdo_firebird_db_handle *)dbh->driver_data;
 
@@ -77,13 +500,11 @@ static int firebird_handle_closer(pdo_dbh_t *dbh) /* {{{ */
 	}
 
 	pefree(H, dbh->is_persistent);
-
-	return 0;
 }
 /* }}} */
 
 /* called by PDO to prepare an SQL query */
-static int firebird_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_len, /* {{{ */
+static bool firebird_handle_preparer(pdo_dbh_t *dbh, zend_string *sql, /* {{{ */
 	pdo_stmt_t *stmt, zval *driver_options)
 {
 	pdo_firebird_db_handle *H = (pdo_firebird_db_handle *)dbh->driver_data;
@@ -103,7 +524,7 @@ static int firebird_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_
 		zend_hash_init(np, 8, NULL, NULL, 0);
 
 		/* allocate and prepare statement */
-		if (!firebird_alloc_prepare_stmt(dbh, sql, sql_len, &num_sqlda, &s, np)) {
+		if (!firebird_alloc_prepare_stmt(dbh, sql, &num_sqlda, &s, np)) {
 			break;
 		}
 
@@ -111,7 +532,6 @@ static int firebird_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_
 		S = ecalloc(1, sizeof(*S)-sizeof(XSQLDA) + XSQLDA_LENGTH(num_sqlda.sqld));
 		S->H = H;
 		S->stmt = s;
-		S->fetch_buf = ecalloc(1,sizeof(char*) * num_sqlda.sqld);
 		S->out_sqlda.version = PDO_FB_SQLDA_VERSION;
 		S->out_sqlda.sqln = stmt->column_count = num_sqlda.sqld;
 		S->named_params = np;
@@ -148,7 +568,7 @@ static int firebird_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_
 		stmt->methods = &firebird_stmt_methods;
 		stmt->supports_placeholders = PDO_PLACEHOLDER_POSITIONAL;
 
-		return 1;
+		return true;
 
 	} while (0);
 
@@ -164,12 +584,12 @@ static int firebird_handle_preparer(pdo_dbh_t *dbh, const char *sql, size_t sql_
 		efree(S);
 	}
 
-	return 0;
+	return false;
 }
 /* }}} */
 
 /* called by PDO to execute a statement that doesn't produce a result set */
-static zend_long firebird_handle_doer(pdo_dbh_t *dbh, const char *sql, size_t sql_len) /* {{{ */
+static zend_long firebird_handle_doer(pdo_dbh_t *dbh, const zend_string *sql) /* {{{ */
 {
 	pdo_firebird_db_handle *H = (pdo_firebird_db_handle *)dbh->driver_data;
 	isc_stmt_handle stmt = PDO_FIREBIRD_HANDLE_INITIALIZER;
@@ -184,7 +604,7 @@ static zend_long firebird_handle_doer(pdo_dbh_t *dbh, const char *sql, size_t sq
 	out_sqlda.sqln = 1;
 
 	/* allocate and prepare statement */
-	if (!firebird_alloc_prepare_stmt(dbh, sql, sql_len, &out_sqlda, &stmt, 0)) {
+	if (!firebird_alloc_prepare_stmt(dbh, sql, &out_sqlda, &stmt, 0)) {
 		return -1;
 	}
 
@@ -240,30 +660,33 @@ free_statement:
 /* }}} */
 
 /* called by the PDO SQL parser to add quotes to values that are copied into SQL */
-static int firebird_handle_quoter(pdo_dbh_t *dbh, const char *unquoted, size_t unquotedlen, /* {{{ */
-	char **quoted, size_t *quotedlen, enum pdo_param_type paramtype)
+static zend_string* firebird_handle_quoter(pdo_dbh_t *dbh, const zend_string *unquoted, enum pdo_param_type paramtype)
 {
-	int qcount = 0;
+	size_t qcount = 0;
 	char const *co, *l, *r;
 	char *c;
+	size_t quotedlen;
+	zend_string *quoted_str;
 
-	if (!unquotedlen) {
-		*quotedlen = 2;
-		*quoted = emalloc(*quotedlen+1);
-		strcpy(*quoted, "''");
-		return 1;
+	if (ZSTR_LEN(unquoted) == 0) {
+		return zend_string_init("''", 2, 0);
 	}
 
 	/* Firebird only requires single quotes to be doubled if string lengths are used */
 	/* count the number of ' characters */
-	for (co = unquoted; (co = strchr(co,'\'')); qcount++, co++);
+	for (co = ZSTR_VAL(unquoted); (co = strchr(co,'\'')); qcount++, co++);
 
-	*quotedlen = unquotedlen + qcount + 2;
-	*quoted = c = emalloc(*quotedlen+1);
+	if (UNEXPECTED(ZSTR_LEN(unquoted) + 2 > ZSTR_MAX_LEN - qcount)) {
+		return NULL;
+	}
+
+	quotedlen = ZSTR_LEN(unquoted) + qcount + 2;
+	quoted_str = zend_string_alloc(quotedlen, 0);
+	c = ZSTR_VAL(quoted_str);
 	*c++ = '\'';
 
 	/* foreach (chunk that ends in a quote) */
-	for (l = unquoted; (r = strchr(l,'\'')); l = r+1) {
+	for (l = ZSTR_VAL(unquoted); (r = strchr(l,'\'')); l = r+1) {
 		strncpy(c, l, r-l+1);
 		c += (r-l+1);
 		/* add the second quote */
@@ -271,20 +694,20 @@ static int firebird_handle_quoter(pdo_dbh_t *dbh, const char *unquoted, size_t u
 	}
 
 	/* copy the remainder */
-	strncpy(c, l, *quotedlen-(c-*quoted)-1);
-	(*quoted)[*quotedlen-1] = '\'';
-	(*quoted)[*quotedlen]   = '\0';
+	strncpy(c, l, quotedlen-(c-ZSTR_VAL(quoted_str))-1);
+	ZSTR_VAL(quoted_str)[quotedlen-1] = '\'';
+	ZSTR_VAL(quoted_str)[quotedlen]   = '\0';
 
-	return 1;
+	return quoted_str;
 }
 /* }}} */
 
 /* called by PDO to start a transaction */
-static int firebird_handle_begin(pdo_dbh_t *dbh) /* {{{ */
+static bool firebird_handle_begin(pdo_dbh_t *dbh) /* {{{ */
 {
 	pdo_firebird_db_handle *H = (pdo_firebird_db_handle *)dbh->driver_data;
 	char tpb[8] = { isc_tpb_version3 }, *ptpb = tpb+1;
-#if abies_0
+#ifdef abies_0
 	if (dbh->transaction_flags & PDO_TRANS_ISOLATION_LEVEL) {
 		if (dbh->transaction_flags & PDO_TRANS_READ_UNCOMMITTED) {
 			/* this is a poor fit, but it's all we have */
@@ -326,48 +749,47 @@ static int firebird_handle_begin(pdo_dbh_t *dbh) /* {{{ */
 #endif
 	if (isc_start_transaction(H->isc_status, &H->tr, 1, &H->db, (unsigned short)(ptpb-tpb), tpb)) {
 		RECORD_ERROR(dbh);
-		return 0;
+		return false;
 	}
-	return 1;
+	return true;
 }
 /* }}} */
 
 /* called by PDO to commit a transaction */
-static int firebird_handle_commit(pdo_dbh_t *dbh) /* {{{ */
+static bool firebird_handle_commit(pdo_dbh_t *dbh) /* {{{ */
 {
 	pdo_firebird_db_handle *H = (pdo_firebird_db_handle *)dbh->driver_data;
 
 	if (isc_commit_transaction(H->isc_status, &H->tr)) {
 		RECORD_ERROR(dbh);
-		return 0;
+		return false;
 	}
-	return 1;
+	return true;
 }
 /* }}} */
 
 /* called by PDO to rollback a transaction */
-static int firebird_handle_rollback(pdo_dbh_t *dbh) /* {{{ */
+static bool firebird_handle_rollback(pdo_dbh_t *dbh) /* {{{ */
 {
 	pdo_firebird_db_handle *H = (pdo_firebird_db_handle *)dbh->driver_data;
 
 	if (isc_rollback_transaction(H->isc_status, &H->tr)) {
 		RECORD_ERROR(dbh);
-		return 0;
+		return false;
 	}
-	return 1;
+	return true;
 }
 /* }}} */
 
 /* used by prepare and exec to allocate a statement handle and prepare the SQL */
-static int firebird_alloc_prepare_stmt(pdo_dbh_t *dbh, const char *sql, size_t sql_len, /* {{{ */
+static int firebird_alloc_prepare_stmt(pdo_dbh_t *dbh, const zend_string *sql,
 	XSQLDA *out_sqlda, isc_stmt_handle *s, HashTable *named_params)
 {
 	pdo_firebird_db_handle *H = (pdo_firebird_db_handle *)dbh->driver_data;
-	char *c, *new_sql, in_quote, in_param, pname[64], *ppname;
-	zend_long l, pindex = -1;
+	char *new_sql;
 
 	/* Firebird allows SQL statements up to 64k, so bail if it doesn't fit */
-	if (sql_len > 65536) {
+	if (ZSTR_LEN(sql) > 65536) {
 		strcpy(dbh->error_code, "01004");
 		return 0;
 	}
@@ -379,7 +801,7 @@ static int firebird_alloc_prepare_stmt(pdo_dbh_t *dbh, const char *sql, size_t s
 		if (!firebird_handle_begin(dbh)) {
 			return 0;
 		}
-		dbh->in_txn = 1;
+		dbh->in_txn = true;
 	}
 
 	/* allocate the statement */
@@ -390,39 +812,12 @@ static int firebird_alloc_prepare_stmt(pdo_dbh_t *dbh, const char *sql, size_t s
 
 	/* in order to support named params, which Firebird itself doesn't,
 	   we need to replace :foo by ?, and store the name we just replaced */
-	new_sql = c = emalloc(sql_len+1);
-
-	for (l = in_quote = in_param = 0; l <= sql_len; ++l) {
-		if ( !(in_quote ^= (sql[l] == '\''))) {
-			if (!in_param) {
-				switch (sql[l]) {
-					case ':':
-						in_param = 1;
-						ppname = pname;
-						*ppname++ = sql[l];
-					case '?':
-						*c++ = '?';
-						++pindex;
-					continue;
-				}
-			} else {
-                                if ((in_param &= ((sql[l] >= 'A' && sql[l] <= 'Z') || (sql[l] >= 'a' && sql[l] <= 'z')
-                                        || (sql[l] >= '0' && sql[l] <= '9') || sql[l] == '_' || sql[l] == '-'))) {
-
-
-					*ppname++ = sql[l];
-					continue;
-				} else {
-					*ppname++ = 0;
-					if (named_params) {
-						zval tmp;
-						ZVAL_LONG(&tmp, pindex);
-						zend_hash_str_update(named_params, pname, (unsigned int)(ppname - pname - 1), &tmp);
-					}
-				}
-			}
-		}
-		*c++ = sql[l];
+	new_sql = emalloc(ZSTR_LEN(sql)+1);
+	new_sql[0] = '\0';
+	if (!preprocess(sql, new_sql, named_params)) {
+		strcpy(dbh->error_code, "07000");
+		efree(new_sql);
+		return 0;
 	}
 
 	/* prepare the statement */
@@ -435,17 +830,19 @@ static int firebird_alloc_prepare_stmt(pdo_dbh_t *dbh, const char *sql, size_t s
 	efree(new_sql);
 	return 1;
 }
-/* }}} */
 
 /* called by PDO to set a driver-specific dbh attribute */
-static int firebird_handle_set_attribute(pdo_dbh_t *dbh, zend_long attr, zval *val) /* {{{ */
+static bool firebird_handle_set_attribute(pdo_dbh_t *dbh, zend_long attr, zval *val) /* {{{ */
 {
 	pdo_firebird_db_handle *H = (pdo_firebird_db_handle *)dbh->driver_data;
+	bool bval;
 
 	switch (attr) {
 		case PDO_ATTR_AUTOCOMMIT:
 			{
-				zend_bool bval = zval_get_long(val)? 1 : 0;
+				if (!pdo_get_bool_param(&bval, val)) {
+					return false;
+				}
 
 				/* ignore if the new value equals the old one */
 				if (dbh->auto_commit ^ bval) {
@@ -454,29 +851,32 @@ static int firebird_handle_set_attribute(pdo_dbh_t *dbh, zend_long attr, zval *v
 							/* turning on auto_commit with an open transaction is illegal, because
 							   we won't know what to do with it */
 							H->last_app_error = "Cannot enable auto-commit while a transaction is already open";
-							return 0;
+							return false;
 						} else {
 							/* close the transaction */
 							if (!firebird_handle_commit(dbh)) {
 								break;
 							}
-							dbh->in_txn = 0;
+							dbh->in_txn = false;
 						}
 					}
 					dbh->auto_commit = bval;
 				}
 			}
-			return 1;
+			return true;
 
 		case PDO_ATTR_FETCH_TABLE_NAMES:
-			H->fetch_table_names = zval_get_long(val)? 1 : 0;
-			return 1;
+			if (!pdo_get_bool_param(&bval, val)) {
+				return false;
+			}
+			H->fetch_table_names = bval;
+			return true;
 
 		case PDO_FB_ATTR_DATE_FORMAT:
 			{
 				zend_string *str = zval_try_get_string(val);
 				if (UNEXPECTED(!str)) {
-					return 0;
+					return false;
 				}
 				if (H->date_format) {
 					efree(H->date_format);
@@ -484,13 +884,13 @@ static int firebird_handle_set_attribute(pdo_dbh_t *dbh, zend_long attr, zval *v
 				spprintf(&H->date_format, 0, "%s", ZSTR_VAL(str));
 				zend_string_release_ex(str, 0);
 			}
-			return 1;
+			return true;
 
 		case PDO_FB_ATTR_TIME_FORMAT:
 			{
 				zend_string *str = zval_try_get_string(val);
 				if (UNEXPECTED(!str)) {
-					return 0;
+					return false;
 				}
 				if (H->time_format) {
 					efree(H->time_format);
@@ -498,13 +898,13 @@ static int firebird_handle_set_attribute(pdo_dbh_t *dbh, zend_long attr, zval *v
 				spprintf(&H->time_format, 0, "%s", ZSTR_VAL(str));
 				zend_string_release_ex(str, 0);
 			}
-			return 1;
+			return true;
 
 		case PDO_FB_ATTR_TIMESTAMP_FORMAT:
 			{
 				zend_string *str = zval_try_get_string(val);
 				if (UNEXPECTED(!str)) {
-					return 0;
+					return false;
 				}
 				if (H->timestamp_format) {
 					efree(H->timestamp_format);
@@ -512,9 +912,9 @@ static int firebird_handle_set_attribute(pdo_dbh_t *dbh, zend_long attr, zval *v
 				spprintf(&H->timestamp_format, 0, "%s", ZSTR_VAL(str));
 				zend_string_release_ex(str, 0);
 			}
-			return 1;
+			return true;
 	}
-	return 0;
+	return false;
 }
 /* }}} */
 
@@ -579,6 +979,8 @@ static int firebird_handle_get_attribute(pdo_dbh_t *dbh, zend_long attr, zval *v
 				ZVAL_STRING(val, tmp);
 				return 1;
 			}
+			/* TODO Check this is correct? */
+			ZEND_FALLTHROUGH;
 
 		case PDO_ATTR_FETCH_TABLE_NAMES:
 			ZVAL_BOOL(val, H->fetch_table_names);
@@ -589,7 +991,7 @@ static int firebird_handle_get_attribute(pdo_dbh_t *dbh, zend_long attr, zval *v
 /* }}} */
 
 /* called by PDO to retrieve driver-specific information about an error that has occurred */
-static int pdo_firebird_fetch_error_func(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval *info) /* {{{ */
+static void pdo_firebird_fetch_error_func(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval *info) /* {{{ */
 {
 	pdo_firebird_db_handle *H = (pdo_firebird_db_handle *)dbh->driver_data;
 	const ISC_STATUS *s = H->isc_status;
@@ -608,7 +1010,6 @@ static int pdo_firebird_fetch_error_func(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval
 		add_next_index_long(info, -999);
 		add_next_index_string(info, const_cast(H->last_app_error));
 	}
-	return 1;
 }
 /* }}} */
 
@@ -624,7 +1025,11 @@ static const struct pdo_dbh_methods firebird_methods = { /* {{{ */
 	NULL, /* last_id not supported */
 	pdo_firebird_fetch_error_func,
 	firebird_handle_get_attribute,
-	NULL /* check_liveness */
+	NULL, /* check_liveness */
+	NULL, /* get driver methods */
+	NULL, /* request shutdown */
+	NULL, /* in transaction, use PDO's internal tracking mechanism */
+	NULL /* get gc */
 };
 /* }}} */
 
@@ -700,7 +1105,7 @@ static int pdo_firebird_handle_factory(pdo_dbh_t *dbh, zval *driver_options) /*
 		char errmsg[512];
 		const ISC_STATUS *s = H->isc_status;
 		fb_interpret(errmsg, sizeof(errmsg),&s);
-		zend_throw_exception_ex(php_pdo_get_exception(), H->isc_status[1], "SQLSTATE[%s] [%d] %s",
+		zend_throw_exception_ex(php_pdo_get_exception(), H->isc_status[1], "SQLSTATE[%s] [%ld] %s",
 				"HY000", H->isc_status[1], errmsg);
 	}
 
