@@ -1,13 +1,11 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
-  +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2018 The PHP Group                                |
+  | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_01.txt                                  |
+  | https://www.php.net/license/3_01.txt                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -16,9 +14,14 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+#ifndef PHP_PDO_OCI_INT_H
+#define PHP_PDO_OCI_INT_H
 
+#include "zend_portability.h"
+
+ZEND_DIAGNOSTIC_IGNORED_START("-Wstrict-prototypes")
 #include <oci.h>
+ZEND_DIAGNOSTIC_IGNORED_END
 
 typedef struct {
 	const char *file;
@@ -85,7 +88,7 @@ typedef struct {
 } pdo_oci_bound_param;
 
 extern const ub4 PDO_OCI_INIT_MODE;
-extern pdo_driver_t pdo_oci_driver;
+extern const pdo_driver_t pdo_oci_driver;
 extern OCIEnv *pdo_oci_Env;
 
 ub4 _oci_error(OCIError *err, pdo_dbh_t *dbh, pdo_stmt_t *stmt, char *what, sword status, int isinit, const char *file, int line);
@@ -93,7 +96,7 @@ ub4 _oci_error(OCIError *err, pdo_dbh_t *dbh, pdo_stmt_t *stmt, char *what, swor
 #define oci_drv_error(w)	_oci_error(H->err, dbh, NULL, w, H->last_err, FALSE, __FILE__, __LINE__)
 #define oci_stmt_error(w)	_oci_error(S->err, stmt->dbh, stmt, w, S->last_err, FALSE, __FILE__, __LINE__)
 
-extern struct pdo_stmt_methods oci_stmt_methods;
+extern const struct pdo_stmt_methods oci_stmt_methods;
 
 /* Default prefetch size in number of rows */
 #define PDO_OCI_PREFETCH_DEFAULT 100
@@ -106,5 +109,8 @@ enum {
 	PDO_OCI_ATTR_ACTION = PDO_ATTR_DRIVER_SPECIFIC,
 	PDO_OCI_ATTR_CLIENT_INFO,
 	PDO_OCI_ATTR_CLIENT_IDENTIFIER,
-	PDO_OCI_ATTR_MODULE
+	PDO_OCI_ATTR_MODULE,
+	PDO_OCI_ATTR_CALL_TIMEOUT
 };
+
+#endif	/* PHP_PDO_OCI_INT_H */
