@@ -7,7 +7,7 @@
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -24,6 +24,8 @@
 #include "php.h"
 #include "php_ini.h"
 #include "php_shmop.h"
+#include "shmop_arginfo.h"
+
 # ifndef PHP_WIN32
 # include <sys/ipc.h>
 # include <sys/shm.h>
@@ -32,70 +34,15 @@
 #endif
 
 
-#if HAVE_SHMOP
+#ifdef HAVE_SHMOP
 
 #include "ext/standard/info.h"
 
-#ifdef ZTS
-int shmop_globals_id;
-#else
-php_shmop_globals shmop_globals;
-#endif
-
-int shm_type;
-
-/* {{{ arginfo */
-ZEND_BEGIN_ARG_INFO_EX(arginfo_shmop_open, 0, 0, 4)
-	ZEND_ARG_INFO(0, key)
-	ZEND_ARG_INFO(0, flags)
-	ZEND_ARG_INFO(0, mode)
-	ZEND_ARG_INFO(0, size)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_shmop_read, 0, 0, 3)
-	ZEND_ARG_INFO(0, shmid)
-	ZEND_ARG_INFO(0, start)
-	ZEND_ARG_INFO(0, count)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_shmop_close, 0, 0, 1)
-	ZEND_ARG_INFO(0, shmid)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_shmop_size, 0, 0, 1)
-	ZEND_ARG_INFO(0, shmid)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_shmop_write, 0, 0, 3)
-	ZEND_ARG_INFO(0, shmid)
-	ZEND_ARG_INFO(0, data)
-	ZEND_ARG_INFO(0, offset)
-ZEND_END_ARG_INFO()
-
-ZEND_BEGIN_ARG_INFO_EX(arginfo_shmop_delete, 0, 0, 1)
-	ZEND_ARG_INFO(0, shmid)
-ZEND_END_ARG_INFO()
-/* }}} */
-
-/* {{{ shmop_functions[]
- */
-static const zend_function_entry shmop_functions[] = {
-	PHP_FE(shmop_open, 		arginfo_shmop_open)
-	PHP_FE(shmop_read, 		arginfo_shmop_read)
-	PHP_FE(shmop_close, 	arginfo_shmop_close)
-	PHP_FE(shmop_size, 		arginfo_shmop_size)
-	PHP_FE(shmop_write, 	arginfo_shmop_write)
-	PHP_FE(shmop_delete, 	arginfo_shmop_delete)
-	PHP_FE_END
-};
-/* }}} */
-
-/* {{{ shmop_module_entry
- */
+/* {{{ shmop_module_entry */
 zend_module_entry shmop_module_entry = {
 	STANDARD_MODULE_HEADER,
 	"shmop",
-	shmop_functions,
+	ext_functions,
 	PHP_MINIT(shmop),
 	NULL,
 	NULL,
@@ -110,29 +57,71 @@ zend_module_entry shmop_module_entry = {
 ZEND_GET_MODULE(shmop)
 #endif
 
-/* {{{ rsclean
- */
-static void rsclean(zend_resource *rsrc)
+typedef struct php_shmop
 {
-	struct php_shmop *shmop = (struct php_shmop *)rsrc->ptr;
+	int shmid;
+	key_t key;
+	int shmflg;
+	int shmatflg;
+	char *addr;
+	zend_long size;
+	zend_object std;
+} php_shmop;
+
+zend_class_entry *shmop_ce;
+static zend_object_handlers shmop_object_handlers;
+
+static inline php_shmop *shmop_from_obj(zend_object *obj)
+{
+	return (php_shmop *)((char *)(obj) - XtOffsetOf(php_shmop, std));
+}
+
+#define Z_SHMOP_P(zv) shmop_from_obj(Z_OBJ_P(zv))
+
+static zend_object *shmop_create_object(zend_class_entry *class_type)
+{
+	php_shmop *intern = zend_object_alloc(sizeof(php_shmop), class_type);
+
+	zend_object_std_init(&intern->std, class_type);
+	object_properties_init(&intern->std, class_type);
+	intern->std.handlers = &shmop_object_handlers;
+
+	return &intern->std;
+}
+
+static zend_function *shmop_get_constructor(zend_object *object)
+{
+	zend_throw_error(NULL, "Cannot directly construct Shmop, use shmop_open() instead");
+	return NULL;
+}
+
+static void shmop_free_obj(zend_object *object)
+{
+	php_shmop *shmop = shmop_from_obj(object);
 
 	shmdt(shmop->addr);
-	efree(shmop);
-}
-/* }}} */
 
-/* {{{ PHP_MINIT_FUNCTION
- */
+	zend_object_std_dtor(&shmop->std);
+}
+
+/* {{{ PHP_MINIT_FUNCTION */
 PHP_MINIT_FUNCTION(shmop)
 {
-	shm_type = zend_register_list_destructors_ex(rsclean, NULL, "shmop", module_number);
+	shmop_ce = register_class_Shmop();
+	shmop_ce->create_object = shmop_create_object;
+
+	memcpy(&shmop_object_handlers, &std_object_handlers, sizeof(zend_object_handlers));
+	shmop_object_handlers.offset = XtOffsetOf(php_shmop, std);
+	shmop_object_handlers.free_obj = shmop_free_obj;
+	shmop_object_handlers.get_constructor = shmop_get_constructor;
+	shmop_object_handlers.clone_obj = NULL;
+	shmop_object_handlers.compare = zend_objects_not_comparable;
 
 	return SUCCESS;
 }
 /* }}} */
 
-/* {{{ PHP_MINFO_FUNCTION
- */
+/* {{{ PHP_MINFO_FUNCTION */
 PHP_MINFO_FUNCTION(shmop)
 {
 	php_info_print_table_start();
@@ -141,28 +130,26 @@ PHP_MINFO_FUNCTION(shmop)
 }
 /* }}} */
 
-/* {{{ proto resource shmop_open(int key, string flags, int mode, int size)
-   gets and attaches a shared memory segment */
+/* {{{ gets and attaches a shared memory segment */
 PHP_FUNCTION(shmop_open)
 {
 	zend_long key, mode, size;
-	struct php_shmop *shmop;
+	php_shmop *shmop;
 	struct shmid_ds shm;
 	char *flags;
 	size_t flags_len;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "lsll", &key, &flags, &flags_len, &mode, &size) == FAILURE) {
-		return;
+		RETURN_THROWS();
 	}
 
 	if (flags_len != 1) {
-		php_error_docref(NULL, E_WARNING, "%s is not a valid flag", flags);
-		RETURN_FALSE;
+		zend_argument_value_error(2, "must be a valid access mode");
+		RETURN_THROWS();
 	}
 
-	shmop = emalloc(sizeof(struct php_shmop));
-	memset(shmop, 0, sizeof(struct php_shmop));
-
+	object_init_ex(return_value, shmop_ce);
+	shmop = Z_SHMOP_P(return_value);
 	shmop->key = key;
 	shmop->shmflg |= mode;
 
@@ -186,74 +173,71 @@ PHP_FUNCTION(shmop_open)
 			*/
 			break;
 		default:
-			php_error_docref(NULL, E_WARNING, "invalid access mode");
+			zend_argument_value_error(2, "must be a valid access mode");
 			goto err;
 	}
 
 	if (shmop->shmflg & IPC_CREAT && shmop->size < 1) {
-		php_error_docref(NULL, E_WARNING, "Shared memory segment size must be greater than zero");
+		zend_argument_value_error(4, "must be greater than 0 for the \"c\" and \"n\" access modes");
 		goto err;
 	}
 
 	shmop->shmid = shmget(shmop->key, shmop->size, shmop->shmflg);
 	if (shmop->shmid == -1) {
-		php_error_docref(NULL, E_WARNING, "unable to attach or create shared memory segment '%s'", strerror(errno));
+		php_error_docref(NULL, E_WARNING, "Unable to attach or create shared memory segment \"%s\"", strerror(errno));
 		goto err;
 	}
 
 	if (shmctl(shmop->shmid, IPC_STAT, &shm)) {
 		/* please do not add coverage here: the segment would be leaked and impossible to delete via php */
-		php_error_docref(NULL, E_WARNING, "unable to get shared memory segment information '%s'", strerror(errno));
+		php_error_docref(NULL, E_WARNING, "Unable to get shared memory segment information \"%s\"", strerror(errno));
 		goto err;
 	}
 
 	if (shm.shm_segsz > ZEND_LONG_MAX) {
-		php_error_docref(NULL, E_WARNING, "shared memory segment too large to attach");
+		zend_argument_value_error(4, "is too large");
 		goto err;
 	}
 
 	shmop->addr = shmat(shmop->shmid, 0, shmop->shmatflg);
 	if (shmop->addr == (char*) -1) {
-		php_error_docref(NULL, E_WARNING, "unable to attach to shared memory segment '%s'", strerror(errno));
+		php_error_docref(NULL, E_WARNING, "Unable to attach to shared memory segment \"%s\"", strerror(errno));
 		goto err;
 	}
 
 	shmop->size = shm.shm_segsz;
+	return;
 
-	RETURN_RES(zend_register_resource(shmop, shm_type));
 err:
-	efree(shmop);
+	zend_object_release(Z_OBJ_P(return_value));
 	RETURN_FALSE;
 }
 /* }}} */
 
-/* {{{ proto string shmop_read(resource shmid, int start, int count)
-   reads from a shm segment */
+/* {{{ reads from a shm segment */
 PHP_FUNCTION(shmop_read)
 {
 	zval *shmid;
 	zend_long start, count;
-	struct php_shmop *shmop;
+	php_shmop *shmop;
 	char *startaddr;
 	int bytes;
 	zend_string *return_string;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "rll", &shmid, &start, &count) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Oll", &shmid, shmop_ce, &start, &count) == FAILURE) {
+		RETURN_THROWS();
 	}
 
-	if ((shmop = (struct php_shmop *)zend_fetch_resource(Z_RES_P(shmid), "shmop", shm_type)) == NULL) {
-		RETURN_FALSE;
-	}
+	shmop = Z_SHMOP_P(shmid);
 
 	if (start < 0 || start > shmop->size) {
-		php_error_docref(NULL, E_WARNING, "start is out of range");
-		RETURN_FALSE;
+		zend_argument_value_error(2, "must be between 0 and the segment size");
+		RETURN_THROWS();
 	}
 
 	if (count < 0 || start > (ZEND_LONG_MAX - count) || start + count > shmop->size) {
-		php_error_docref(NULL, E_WARNING, "count is out of range");
-		RETURN_FALSE;
+		zend_argument_value_error(3, "is out of range");
+		RETURN_THROWS();
 	}
 
 	startaddr = shmop->addr + start;
@@ -265,71 +249,56 @@ PHP_FUNCTION(shmop_read)
 }
 /* }}} */
 
-/* {{{ proto void shmop_close(resource shmid)
-   closes a shared memory segment */
+/* {{{ used to close a shared memory segment; now a NOP */
 PHP_FUNCTION(shmop_close)
 {
 	zval *shmid;
-	struct php_shmop *shmop;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "r", &shmid) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &shmid, shmop_ce) == FAILURE) {
+		RETURN_THROWS();
 	}
-
-
-	if ((shmop = (struct php_shmop *)zend_fetch_resource(Z_RES_P(shmid), "shmop", shm_type)) == NULL) {
-		RETURN_FALSE;
-	}
-
-	zend_list_close(Z_RES_P(shmid));
 }
 /* }}} */
 
-/* {{{ proto int shmop_size(resource shmid)
-   returns the shm size */
+/* {{{ returns the shm size */
 PHP_FUNCTION(shmop_size)
 {
 	zval *shmid;
-	struct php_shmop *shmop;
+	php_shmop *shmop;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "r", &shmid) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &shmid, shmop_ce) == FAILURE) {
+		RETURN_THROWS();
 	}
 
-	if ((shmop = (struct php_shmop *)zend_fetch_resource(Z_RES_P(shmid), "shmop", shm_type)) == NULL) {
-		RETURN_FALSE;
-	}
+	shmop = Z_SHMOP_P(shmid);
 
 	RETURN_LONG(shmop->size);
 }
 /* }}} */
 
-/* {{{ proto int shmop_write(resource shmid, string data, int offset)
-   writes to a shared memory segment */
+/* {{{ writes to a shared memory segment */
 PHP_FUNCTION(shmop_write)
 {
-	struct php_shmop *shmop;
+	php_shmop *shmop;
 	zend_long writesize;
 	zend_long offset;
 	zend_string *data;
 	zval *shmid;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "rSl", &shmid, &data, &offset) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "OSl", &shmid, shmop_ce, &data, &offset) == FAILURE) {
+		RETURN_THROWS();
 	}
 
-	if ((shmop = (struct php_shmop *)zend_fetch_resource(Z_RES_P(shmid), "shmop", shm_type)) == NULL) {
-		RETURN_FALSE;
-	}
+	shmop = Z_SHMOP_P(shmid);
 
 	if ((shmop->shmatflg & SHM_RDONLY) == SHM_RDONLY) {
-		php_error_docref(NULL, E_WARNING, "trying to write to a read only segment");
-		RETURN_FALSE;
+		zend_throw_error(NULL, "Read-only segment cannot be written");
+		RETURN_THROWS();
 	}
 
 	if (offset < 0 || offset > shmop->size) {
-		php_error_docref(NULL, E_WARNING, "offset out of range");
-		RETURN_FALSE;
+		zend_argument_value_error(3, "is out of range");
+		RETURN_THROWS();
 	}
 
 	writesize = ((zend_long)ZSTR_LEN(data) < shmop->size - offset) ? (zend_long)ZSTR_LEN(data) : shmop->size - offset;
@@ -339,23 +308,20 @@ PHP_FUNCTION(shmop_write)
 }
 /* }}} */
 
-/* {{{ proto bool shmop_delete(resource shmid)
-   mark segment for deletion */
+/* {{{ mark segment for deletion */
 PHP_FUNCTION(shmop_delete)
 {
 	zval *shmid;
-	struct php_shmop *shmop;
+	php_shmop *shmop;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "r", &shmid) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &shmid, shmop_ce) == FAILURE) {
+		RETURN_THROWS();
 	}
 
-	if ((shmop = (struct php_shmop *)zend_fetch_resource(Z_RES_P(shmid), "shmop", shm_type)) == NULL) {
-		RETURN_FALSE;
-	}
+	shmop = Z_SHMOP_P(shmid);
 
 	if (shmctl(shmop->shmid, IPC_RMID, NULL)) {
-		php_error_docref(NULL, E_WARNING, "can't mark segment for deletion (are you the owner?)");
+		php_error_docref(NULL, E_WARNING, "Can't mark segment for deletion (are you the owner?)");
 		RETURN_FALSE;
 	}
 
