@@ -5,7 +5,7 @@
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -47,6 +47,10 @@
 #endif
 #if HAVE_RESOLV_H
 #include <resolv.h>
+#if defined(__HAIKU__)
+extern void __res_ndestroy(res_state statp);
+#define res_ndestroy __res_ndestroy
+#endif
 #endif
 #ifdef HAVE_DNS_H
 #include <dns.h>
@@ -169,20 +173,32 @@ PHP_FUNCTION(gethostbyaddr)
 static zend_string *php_gethostbyaddr(char *ip)
 {
 #if HAVE_IPV6 && HAVE_INET_PTON
-	struct in6_addr addr6;
-#endif
+	struct sockaddr_in sa4;
+	struct sockaddr_in6 sa6;
+	char out[NI_MAXHOST];
+	memset(&sa4, 0, sizeof(struct sockaddr_in));
+	memset(&sa6, 0, sizeof(struct sockaddr_in6));
+
+	if (inet_pton(AF_INET6, ip, &sa6.sin6_addr)) {
+		sa6.sin6_family = AF_INET6;
+
+		if (getnameinfo((struct sockaddr *)&sa6, sizeof(sa6), out, sizeof(out), NULL, 0, NI_NAMEREQD) != 0) {
+			return zend_string_init(ip, strlen(ip), 0);
+		}
+		return zend_string_init(out, strlen(out), 0);
+	} else if (inet_pton(AF_INET, ip, &sa4.sin_addr)) {
+		sa4.sin_family = AF_INET;
+
+		if (getnameinfo((struct sockaddr *)&sa4, sizeof(sa4), out, sizeof(out), NULL, 0, NI_NAMEREQD) != 0) {
+			return zend_string_init(ip, strlen(ip), 0);
+		}
+		return zend_string_init(out, strlen(out), 0);
+	}
+	return NULL; /* not a valid IP */
+#else
 	struct in_addr addr;
 	struct hostent *hp;
 
-#if HAVE_IPV6 && HAVE_INET_PTON
-	if (inet_pton(AF_INET6, ip, &addr6)) {
-		hp = gethostbyaddr((char *) &addr6, sizeof(addr6), AF_INET6);
-	} else if (inet_pton(AF_INET, ip, &addr)) {
-		hp = gethostbyaddr((char *) &addr, sizeof(addr), AF_INET);
-	} else {
-		return NULL;
-	}
-#else
 	addr.s_addr = inet_addr(ip);
 
 	if (addr.s_addr == -1) {
@@ -190,13 +206,13 @@ static zend_string *php_gethostbyaddr(char *ip)
 	}
 
 	hp = gethostbyaddr((char *) &addr, sizeof(addr), AF_INET);
-#endif
 
 	if (!hp || hp->h_name == NULL || hp->h_name[0] == '\0') {
 		return zend_string_init(ip, strlen(ip), 0);
 	}
 
 	return zend_string_init(hp->h_name, strlen(hp->h_name), 0);
+#endif
 }
 /* }}} */
 
@@ -228,6 +244,9 @@ PHP_FUNCTION(gethostbynamel)
 	struct hostent *hp;
 	struct in_addr in;
 	int i;
+#ifdef HAVE_INET_NTOP
+	char addr4[INET_ADDRSTRLEN];
+#endif
 
 	ZEND_PARSE_PARAMETERS_START(1, 1)
 		Z_PARAM_PATH(hostname, hostname_len)
@@ -255,7 +274,11 @@ PHP_FUNCTION(gethostbynamel)
 		}
 
 		in = *h_addr_entry;
+#ifdef HAVE_INET_NTOP
+		add_next_index_string(return_value, inet_ntop(AF_INET, &in, addr4, INET_ADDRSTRLEN));
+#else
 		add_next_index_string(return_value, inet_ntoa(in));
+#endif
 	}
 }
 /* }}} */
@@ -266,7 +289,10 @@ static zend_string *php_gethostbyname(char *name)
 	struct hostent *hp;
 	struct in_addr *h_addr_0; /* Don't call this h_addr, it's a macro! */
 	struct in_addr in;
-	char *address;
+#ifdef HAVE_INET_NTOP
+	char addr4[INET_ADDRSTRLEN];
+#endif
+	const char *address;
 
 	hp = php_network_gethostbyname(name);
 	if (!hp) {
@@ -281,7 +307,11 @@ static zend_string *php_gethostbyname(char *name)
 
 	memcpy(&in.s_addr, h_addr_0, sizeof(in.s_addr));
 
+#ifdef HAVE_INET_NTOP
+	address = inet_ntop(AF_INET, &in, addr4, INET_ADDRSTRLEN);
+#else
 	address = inet_ntoa(in);
+#endif
 	return zend_string_init(address, strlen(address), 0);
 }
 /* }}} */
@@ -355,8 +385,9 @@ PHP_FUNCTION(dns_check_record)
 {
 	HEADER *hp;
 	querybuf answer;
-	char *hostname, *rectype = NULL;
-	size_t hostname_len, rectype_len = 0;
+	char *hostname;
+	size_t hostname_len;
+	zend_string *rectype = NULL;
 	int type = DNS_T_MX, i;
 #if defined(HAVE_DNS_SEARCH)
 	struct sockaddr_storage from;
@@ -368,9 +399,9 @@ PHP_FUNCTION(dns_check_record)
 #endif
 
 	ZEND_PARSE_PARAMETERS_START(1, 2)
-		Z_PARAM_STRING(hostname, hostname_len)
+		Z_PARAM_PATH(hostname, hostname_len)
 		Z_PARAM_OPTIONAL
-		Z_PARAM_STRING(rectype, rectype_len)
+		Z_PARAM_STR(rectype)
 	ZEND_PARSE_PARAMETERS_END();
 
 	if (hostname_len == 0) {
@@ -379,19 +410,19 @@ PHP_FUNCTION(dns_check_record)
 	}
 
 	if (rectype) {
-		if (!strcasecmp("A",     rectype)) type = DNS_T_A;
-		else if (!strcasecmp("NS",    rectype)) type = DNS_T_NS;
-		else if (!strcasecmp("MX",    rectype)) type = DNS_T_MX;
-		else if (!strcasecmp("PTR",   rectype)) type = DNS_T_PTR;
-		else if (!strcasecmp("ANY",   rectype)) type = DNS_T_ANY;
-		else if (!strcasecmp("SOA",   rectype)) type = DNS_T_SOA;
-		else if (!strcasecmp("CAA",   rectype)) type = DNS_T_CAA;
-		else if (!strcasecmp("TXT",   rectype)) type = DNS_T_TXT;
-		else if (!strcasecmp("CNAME", rectype)) type = DNS_T_CNAME;
-		else if (!strcasecmp("AAAA",  rectype)) type = DNS_T_AAAA;
-		else if (!strcasecmp("SRV",   rectype)) type = DNS_T_SRV;
-		else if (!strcasecmp("NAPTR", rectype)) type = DNS_T_NAPTR;
-		else if (!strcasecmp("A6",    rectype)) type = DNS_T_A6;
+		if (zend_string_equals_literal_ci(rectype, "A")) type = DNS_T_A;
+		else if (zend_string_equals_literal_ci(rectype, "NS")) type = DNS_T_NS;
+		else if (zend_string_equals_literal_ci(rectype, "MX")) type = DNS_T_MX;
+		else if (zend_string_equals_literal_ci(rectype, "PTR")) type = DNS_T_PTR;
+		else if (zend_string_equals_literal_ci(rectype, "ANY")) type = DNS_T_ANY;
+		else if (zend_string_equals_literal_ci(rectype, "SOA")) type = DNS_T_SOA;
+		else if (zend_string_equals_literal_ci(rectype, "CAA")) type = DNS_T_CAA;
+		else if (zend_string_equals_literal_ci(rectype, "TXT")) type = DNS_T_TXT;
+		else if (zend_string_equals_literal_ci(rectype, "CNAME")) type = DNS_T_CNAME;
+		else if (zend_string_equals_literal_ci(rectype, "AAAA")) type = DNS_T_AAAA;
+		else if (zend_string_equals_literal_ci(rectype, "SRV")) type = DNS_T_SRV;
+		else if (zend_string_equals_literal_ci(rectype, "NAPTR")) type = DNS_T_NAPTR;
+		else if (zend_string_equals_literal_ci(rectype, "A6")) type = DNS_T_A6;
 		else {
 			zend_argument_value_error(2, "must be a valid DNS record type");
 			RETURN_THROWS();
@@ -404,8 +435,8 @@ PHP_FUNCTION(dns_check_record)
 		RETURN_FALSE;
 	}
 #elif defined(HAVE_RES_NSEARCH)
-    memset(&state, 0, sizeof(state));
-    if (res_ninit(handle)) {
+	memset(&state, 0, sizeof(state));
+	if (res_ninit(handle)) {
 			RETURN_FALSE;
 	}
 #else
@@ -456,11 +487,11 @@ static u_char *php_parserr(u_char *cp, u_char *end, querybuf *answer, int type_t
 	GETLONG(ttl, cp);
 	GETSHORT(dlen, cp);
 	CHECKCP(dlen);
-    if (dlen == 0) {
-        /* No data in the response - nothing to do */
-        return NULL;
-    }
-    if (type_to_fetch != DNS_T_ANY && type != type_to_fetch) {
+	if (dlen == 0) {
+		/* No data in the response - nothing to do */
+		return NULL;
+	}
+	if (type_to_fetch != DNS_T_ANY && type != type_to_fetch) {
 		cp += dlen;
 		return cp;
 	}
@@ -497,17 +528,17 @@ static u_char *php_parserr(u_char *cp, u_char *end, querybuf *answer, int type_t
 			add_assoc_string(subarray, "type", "MX");
 			GETSHORT(n, cp);
 			add_assoc_long(subarray, "pri", n);
-			/* no break; */
+			ZEND_FALLTHROUGH;
 		case DNS_T_CNAME:
 			if (type == DNS_T_CNAME) {
 				add_assoc_string(subarray, "type", "CNAME");
 			}
-			/* no break; */
+			ZEND_FALLTHROUGH;
 		case DNS_T_NS:
 			if (type == DNS_T_NS) {
 				add_assoc_string(subarray, "type", "NS");
 			}
-			/* no break; */
+			ZEND_FALLTHROUGH;
 		case DNS_T_PTR:
 			if (type == DNS_T_PTR) {
 				add_assoc_string(subarray, "type", "PTR");
@@ -812,10 +843,10 @@ PHP_FUNCTION(dns_get_record)
 	u_char *cp = NULL, *end = NULL;
 	int n, qd, an, ns = 0, ar = 0;
 	int type, first_query = 1, store_results = 1;
-	zend_bool raw = 0;
+	bool raw = 0;
 
 	ZEND_PARSE_PARAMETERS_START(1, 5)
-		Z_PARAM_STRING(hostname, hostname_len)
+		Z_PARAM_PATH(hostname, hostname_len)
 		Z_PARAM_OPTIONAL
 		Z_PARAM_LONG(type_param)
 		Z_PARAM_ZVAL(authns)
@@ -1053,7 +1084,7 @@ PHP_FUNCTION(dns_get_mx)
 #endif
 
 	ZEND_PARSE_PARAMETERS_START(2, 3)
-		Z_PARAM_STRING(hostname, hostname_len)
+		Z_PARAM_PATH(hostname, hostname_len)
 		Z_PARAM_ZVAL(mx_list)
 		Z_PARAM_OPTIONAL
 		Z_PARAM_ZVAL(weight_list)
@@ -1077,8 +1108,8 @@ PHP_FUNCTION(dns_get_mx)
 		RETURN_FALSE;
 	}
 #elif defined(HAVE_RES_NSEARCH)
-    memset(&state, 0, sizeof(state));
-    if (res_ninit(handle)) {
+	memset(&state, 0, sizeof(state));
+	if (res_ninit(handle)) {
 			RETURN_FALSE;
 	}
 #else
