@@ -1,13 +1,11 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -23,15 +21,14 @@
 #include "php_string.h"
 
 /* Simple soundex algorithm as described by Knuth in TAOCP, vol 3 */
-/* {{{ proto string soundex(string str)
-   Calculate the soundex key of a string */
+/* {{{ Calculate the soundex key of a string */
 PHP_FUNCTION(soundex)
 {
 	char	*str;
 	size_t	i, _small, str_len, code, last;
 	char	soundex[4 + 1];
 
-	static char soundex_table[26] =
+	static const char soundex_table[26] =
 	{0,							/* A */
 	 '1',						/* B */
 	 '2',						/* C */
@@ -63,10 +60,6 @@ PHP_FUNCTION(soundex)
 		Z_PARAM_STRING(str, str_len)
 	ZEND_PARSE_PARAMETERS_END();
 
-	if (str_len == 0) {
-		RETURN_FALSE;
-	}
-
 	/* build soundex string */
 	last = -1;
 	for (i = 0, _small = 0; i < str_len && _small < 4; i++) {
