@@ -5,7 +5,7 @@
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_01.txt                                  |
+  | https://www.php.net/license/3_01.txt                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -85,7 +85,7 @@ static int xmlreader_property_reader(xmlreader_object *obj, xmlreader_prop_handl
 			if (hnd->read_int_func) {
 				retint = hnd->read_int_func(obj->ptr);
 				if (retint == -1) {
-					php_error_docref(NULL, E_WARNING, "Internal libxml error returned");
+					zend_throw_error(NULL, "Failed to read property due to libxml error");
 					return FAILURE;
 				}
 			}
@@ -100,15 +100,13 @@ static int xmlreader_property_reader(xmlreader_object *obj, xmlreader_prop_handl
 				ZVAL_EMPTY_STRING(rv);
 			}
 			break;
-		/* this IS_FALSE actually means it's a BOOL type */
-		case IS_FALSE:
+		case _IS_BOOL:
 			ZVAL_BOOL(rv, retint);
 			break;
 		case IS_LONG:
 			ZVAL_LONG(rv, retint);
 			break;
-		default:
-			ZVAL_NULL(rv);
+		EMPTY_SWITCH_DEFAULT_CASE()
 	}
 
 	return SUCCESS;
@@ -213,6 +211,9 @@ char *_xmlreader_get_valid_file_path(char *source, char *resolved_path, int reso
 	int isFileUri = 0;
 
 	uri = xmlCreateURI();
+	if (uri == NULL) {
+		return NULL;
+	}
 	escsource = xmlURIEscapeStr((xmlChar *)source, (xmlChar *)":");
 	xmlParseURIReference(uri, (const char *)escsource);
 	xmlFree(escsource);
@@ -284,6 +285,7 @@ static xmlRelaxNGPtr _xmlreader_get_relaxNG(char *source, size_t source_len, siz
 		return NULL;
 	}
 
+	PHP_LIBXML_SANITIZE_GLOBALS(parse);
 	if (error_func || warn_func) {
 		xmlRelaxNGSetParserErrors(parser,
 			(xmlRelaxNGValidityErrorFunc) error_func,
@@ -292,6 +294,7 @@ static xmlRelaxNGPtr _xmlreader_get_relaxNG(char *source, size_t source_len, siz
 	}
 	sptr = xmlRelaxNGParse(parser);
 	xmlRelaxNGFreeParserCtxt(parser);
+	PHP_LIBXML_RESTORE_GLOBALS(parse);
 
 	return sptr;
 }
@@ -332,23 +335,21 @@ void xmlreader_objects_clone(void *object, void **object_clone)
 
 /* {{{ xmlreader_free_resources */
 static void xmlreader_free_resources(xmlreader_object *intern) {
-	if (intern) {
-		if (intern->input) {
-			xmlFreeParserInputBuffer(intern->input);
-			intern->input = NULL;
-		}
-
-		if (intern->ptr) {
-			xmlFreeTextReader(intern->ptr);
-			intern->ptr = NULL;
-		}
-#ifdef LIBXML_SCHEMAS_ENABLED
-		if (intern->schema) {
-			xmlRelaxNGFree((xmlRelaxNGPtr) intern->schema);
-			intern->schema = NULL;
-		}
-#endif
+	if (intern->input) {
+		xmlFreeParserInputBuffer(intern->input);
+		intern->input = NULL;
 	}
+
+	if (intern->ptr) {
+		xmlFreeTextReader(intern->ptr);
+		intern->ptr = NULL;
+	}
+#ifdef LIBXML_SCHEMAS_ENABLED
+	if (intern->schema) {
+		xmlRelaxNGFree((xmlRelaxNGPtr) intern->schema);
+		intern->schema = NULL;
+	}
+#endif
 }
 /* }}} */
 
@@ -398,7 +399,7 @@ static void php_xmlreader_string_arg(INTERNAL_FUNCTION_PARAMETERS, xmlreader_rea
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		retchar = (char *)internal_function(intern->ptr, (const unsigned char *)name);
 	}
 	if (retchar) {
@@ -424,7 +425,7 @@ static void php_xmlreader_no_arg(INTERNAL_FUNCTION_PARAMETERS, xmlreader_read_in
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		retval = internal_function(intern->ptr);
 		if (retval == 1) {
 			RETURN_TRUE;
@@ -452,7 +453,7 @@ static void php_xmlreader_no_arg_string(INTERNAL_FUNCTION_PARAMETERS, xmlreader_
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		retchar = (char *)internal_function(intern->ptr);
 	}
 	if (retchar) {
@@ -487,7 +488,7 @@ static void php_xmlreader_set_relaxng_schema(INTERNAL_FUNCTION_PARAMETERS, int t
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		if (source) {
 			schema =  _xmlreader_get_relaxNG(source, source_len, type, NULL, NULL);
 			if (schema) {
@@ -564,7 +565,7 @@ PHP_METHOD(XMLReader, getAttributeNo)
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		retchar = (char *)xmlTextReaderGetAttributeNo(intern->ptr, attr_pos);
 	}
 	if (retchar) {
@@ -599,7 +600,7 @@ PHP_METHOD(XMLReader, getAttributeNs)
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		retchar = (char *)xmlTextReaderGetAttributeNs(intern->ptr, (xmlChar *)name, (xmlChar *)ns_uri);
 	}
 	if (retchar) {
@@ -614,7 +615,6 @@ PHP_METHOD(XMLReader, getParserProperty)
 {
 	zval *id;
 	zend_long property;
-	int retval = -1;
 	xmlreader_object *intern;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &property) == FAILURE) {
@@ -624,9 +624,12 @@ PHP_METHOD(XMLReader, getParserProperty)
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
-		retval = xmlTextReaderGetParserProp(intern->ptr,property);
+	if (!intern || !intern->ptr) {
+		zend_throw_error(NULL, "Cannot access parser properties before loading data");
+		RETURN_THROWS();
 	}
+
+	int retval = xmlTextReaderGetParserProp(intern->ptr,property);
 	if (retval == -1) {
 		zend_argument_value_error(1, "must be a valid parser property");
 		RETURN_THROWS();
@@ -673,7 +676,7 @@ PHP_METHOD(XMLReader, moveToAttribute)
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		retval = xmlTextReaderMoveToAttribute(intern->ptr, (xmlChar *)name);
 		if (retval == 1) {
 			RETURN_TRUE;
@@ -700,7 +703,7 @@ PHP_METHOD(XMLReader, moveToAttributeNo)
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		retval = xmlTextReaderMoveToAttributeNo(intern->ptr, attr_pos);
 		if (retval == 1) {
 			RETURN_TRUE;
@@ -738,7 +741,7 @@ PHP_METHOD(XMLReader, moveToAttributeNs)
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		retval = xmlTextReaderMoveToAttributeNs(intern->ptr, (xmlChar *)name, (xmlChar *)ns_uri);
 		if (retval == 1) {
 			RETURN_TRUE;
@@ -783,7 +786,7 @@ PHP_METHOD(XMLReader, read)
 
 	id = ZEND_THIS;
 	intern = Z_XMLREADER_P(id);
-	if (intern == NULL || intern->ptr == NULL) {
+	if (!intern->ptr) {
 		zend_throw_error(NULL, "Data must be loaded before reading");
 		RETURN_THROWS();
 	}
@@ -812,7 +815,7 @@ PHP_METHOD(XMLReader, next)
 
 	id = ZEND_THIS;
 	intern = Z_XMLREADER_P(id);
-	if (intern != NULL && intern->ptr != NULL) {
+	if (intern->ptr) {
 		retval = xmlTextReaderNext(intern->ptr);
 		while (name != NULL && retval == 1) {
 			if (xmlStrEqual(xmlTextReaderConstLocalName(intern->ptr), (xmlChar *)name)) {
@@ -849,12 +852,9 @@ PHP_METHOD(XMLReader, open)
 
 	id = getThis();
 	if (id != NULL) {
-		if (! instanceof_function(Z_OBJCE_P(id), xmlreader_class_entry)) {
-			id = NULL;
-		} else {
-			intern = Z_XMLREADER_P(id);
-			xmlreader_free_resources(intern);
-		}
+		ZEND_ASSERT(instanceof_function(Z_OBJCE_P(id), xmlreader_class_entry));
+		intern = Z_XMLREADER_P(id);
+		xmlreader_free_resources(intern);
 	}
 
 	if (!source_len) {
@@ -870,7 +870,9 @@ PHP_METHOD(XMLReader, open)
 	valid_file = _xmlreader_get_valid_file_path(source, resolved_path, MAXPATHLEN );
 
 	if (valid_file) {
+		PHP_LIBXML_SANITIZE_GLOBALS(reader_for_file);
 		reader = xmlReaderForFile(valid_file, encoding, options);
+		PHP_LIBXML_RESTORE_GLOBALS(reader_for_file);
 	}
 
 	if (reader == NULL) {
@@ -943,7 +945,9 @@ PHP_METHOD(XMLReader, setSchema)
 
 	intern = Z_XMLREADER_P(id);
 	if (intern && intern->ptr) {
+		PHP_LIBXML_SANITIZE_GLOBALS(schema);
 		retval = xmlTextReaderSchemaValidate(intern->ptr, source);
+		PHP_LIBXML_RESTORE_GLOBALS(schema);
 
 		if (retval == 0) {
 			RETURN_TRUE;
@@ -968,8 +972,7 @@ PHP_METHOD(XMLReader, setParserProperty)
 {
 	zval *id;
 	zend_long property;
-	int retval = -1;
-	zend_bool value;
+	bool value;
 	xmlreader_object *intern;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "lb", &property, &value) == FAILURE) {
@@ -979,9 +982,12 @@ PHP_METHOD(XMLReader, setParserProperty)
 	id = ZEND_THIS;
 
 	intern = Z_XMLREADER_P(id);
-	if (intern && intern->ptr) {
-		retval = xmlTextReaderSetParserProp(intern->ptr,property, value);
+	if (!intern || !intern->ptr) {
+		zend_throw_error(NULL, "Cannot access parser properties before loading data");
+		RETURN_THROWS();
 	}
+
+	int retval = xmlTextReaderSetParserProp(intern->ptr,property, value);
 	if (retval == -1) {
 		zend_argument_value_error(1, "must be a valid parser property");
 		RETURN_THROWS();
@@ -1020,7 +1026,7 @@ PHP_METHOD(XMLReader, XML)
 	xmlreader_object *intern = NULL;
 	char *source, *uri = NULL, *encoding = NULL;
 	int resolved_path_len, ret = 0;
-	char *directory=NULL, resolved_path[MAXPATHLEN];
+	char *directory=NULL, resolved_path[MAXPATHLEN + 1];
 	xmlParserInputBufferPtr inputbfr;
 	xmlTextReaderPtr reader;
 
@@ -1029,10 +1035,8 @@ PHP_METHOD(XMLReader, XML)
 	}
 
 	id = getThis();
-	if (id != NULL && ! instanceof_function(Z_OBJCE_P(id), xmlreader_class_entry)) {
-		id = NULL;
-	}
 	if (id != NULL) {
+		ZEND_ASSERT(instanceof_function(Z_OBJCE_P(id), xmlreader_class_entry));
 		intern = Z_XMLREADER_P(id);
 		xmlreader_free_resources(intern);
 	}
@@ -1049,7 +1053,7 @@ PHP_METHOD(XMLReader, XML)
 
 	inputbfr = xmlParserInputBufferCreateMem(source, source_len, XML_CHAR_ENCODING_NONE);
 
-    if (inputbfr != NULL) {
+	if (inputbfr != NULL) {
 /* Get the URI of the current script so that we can set the base directory in libxml */
 #ifdef HAVE_GETCWD
 		directory = VCWD_GETCWD(resolved_path, MAXPATHLEN);
@@ -1064,6 +1068,7 @@ PHP_METHOD(XMLReader, XML)
 			}
 			uri = (char *) xmlCanonicPath((const xmlChar *) resolved_path);
 		}
+		PHP_LIBXML_SANITIZE_GLOBALS(text_reader);
 		reader = xmlNewTextReader(inputbfr, uri);
 
 		if (reader != NULL) {
@@ -1082,9 +1087,11 @@ PHP_METHOD(XMLReader, XML)
 					xmlFree(uri);
 				}
 
+				PHP_LIBXML_RESTORE_GLOBALS(text_reader);
 				return;
 			}
 		}
+		PHP_LIBXML_RESTORE_GLOBALS(text_reader);
 	}
 
 	if (uri) {
@@ -1122,7 +1129,7 @@ PHP_METHOD(XMLReader, expand)
 
 	intern = Z_XMLREADER_P(id);
 
-	if (intern && intern->ptr) {
+	if (intern->ptr) {
 		node = xmlTextReaderExpand(intern->ptr);
 
 		if (node == NULL) {
@@ -1157,11 +1164,8 @@ PHP_METHOD(XMLReader, expand)
 PHP_MINIT_FUNCTION(xmlreader)
 {
 
-	zend_class_entry ce;
-
 	memcpy(&xmlreader_object_handlers, &std_object_handlers, sizeof(zend_object_handlers));
 	xmlreader_object_handlers.offset = XtOffsetOf(xmlreader_object, std);
-	xmlreader_object_handlers.dtor_obj = zend_objects_destroy_object;
 	xmlreader_object_handlers.free_obj = xmlreader_objects_free_storage;
 	xmlreader_object_handlers.read_property = xmlreader_read_property;
 	xmlreader_object_handlers.write_property = xmlreader_write_property;
@@ -1169,9 +1173,8 @@ PHP_MINIT_FUNCTION(xmlreader)
 	xmlreader_object_handlers.get_method = xmlreader_get_method;
 	xmlreader_object_handlers.clone_obj = NULL;
 
-	INIT_CLASS_ENTRY(ce, "XMLReader", class_XMLReader_methods);
-	ce.create_object = xmlreader_objects_new;
-	xmlreader_class_entry = zend_register_internal_class(&ce);
+	xmlreader_class_entry = register_class_XMLReader();
+	xmlreader_class_entry->create_object = xmlreader_objects_new;
 
 	memcpy(&xmlreader_open_fn, zend_hash_str_find_ptr(&xmlreader_class_entry->function_table, "open", sizeof("open")-1), sizeof(zend_internal_function));
 	xmlreader_open_fn.fn_flags &= ~ZEND_ACC_STATIC;
@@ -1182,10 +1185,10 @@ PHP_MINIT_FUNCTION(xmlreader)
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "attributeCount", xmlTextReaderAttributeCount, NULL, IS_LONG);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "baseURI", NULL, xmlTextReaderConstBaseUri, IS_STRING);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "depth", xmlTextReaderDepth, NULL, IS_LONG);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "hasAttributes", xmlTextReaderHasAttributes, NULL, IS_FALSE);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "hasValue", xmlTextReaderHasValue, NULL, IS_FALSE);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "isDefault", xmlTextReaderIsDefault, NULL, IS_FALSE);
-	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "isEmptyElement", xmlTextReaderIsEmptyElement, NULL, IS_FALSE);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "hasAttributes", xmlTextReaderHasAttributes, NULL, _IS_BOOL);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "hasValue", xmlTextReaderHasValue, NULL, _IS_BOOL);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "isDefault", xmlTextReaderIsDefault, NULL, _IS_BOOL);
+	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "isEmptyElement", xmlTextReaderIsEmptyElement, NULL, _IS_BOOL);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "localName", NULL, xmlTextReaderConstLocalName, IS_STRING);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "name", NULL, xmlTextReaderConstName, IS_STRING);
 	xmlreader_register_prop_handler(&xmlreader_prop_handlers, "namespaceURI", NULL, xmlTextReaderConstNamespaceUri, IS_STRING);
