@@ -5,7 +5,7 @@
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -68,9 +68,9 @@ void php_ap2_register_hook(apr_pool_t *p);
 #define APR_ARRAY_FOREACH_CLOSE() }}
 
 typedef struct {
-	zend_bool engine;
-	zend_bool xbithack;
-	zend_bool last_modified;
+	bool engine;
+	bool xbithack;
+	bool last_modified;
 } php_apache2_info_struct;
 
 extern zend_module_entry apache2_module_entry;
