@@ -1,5 +1,3 @@
-
-	/* $Id: fpm_sockets.h,v 1.12 2008/08/26 15:09:15 anight Exp $ */
 	/* (c) 2007,2008 Andrei Nigmatulin */
 
 #ifndef FPM_MISC_H
@@ -16,7 +14,7 @@
 /*
   On FreeBSD and OpenBSD, backlog negative values are truncated to SOMAXCONN
 */
-#if (__FreeBSD__) || (__OpenBSD__)
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
 #define FPM_BACKLOG_DEFAULT -1
 #else
 #define FPM_BACKLOG_DEFAULT 511
@@ -26,7 +24,7 @@
 #define FPM_ENV_SOCKET_SET_SIZE 128
 
 enum fpm_address_domain fpm_sockets_domain_from_address(char *addr);
-int fpm_sockets_init_main();
+int fpm_sockets_init_main(void);
 int fpm_socket_get_listening_queue(int sock, unsigned *cur_lq, unsigned *max_lq);
 int fpm_socket_unix_test_connect(struct sockaddr_un *sock, size_t socklen);
 
